function convertAnaglyph()
%CONVERTANAGLYPH Converts a stereo image pair to optimal colors for red/cyan viewing
% When viewing result, red filter goes over left eye, cyan filter goes over right eye
% Author: Luke Gane
% Last updated: 2015-02-12

% Optimal colors depend on display screen and may not exist for some screen configurations
colorBackground = [255 118 118];
colorLeft = [0 118 118];
colorRight = [255 0 0];

filterSpecCellArray = {'*.png;*.tif;*.jpg;*.gif','Image files (*.png, *.tif, *.jpg, *.gif)'; '*.*','All Files (*.*)'};

while true
	
	[fileNameLeft, filePathLeft] = uigetfile(filterSpecCellArray, 'Select the image to be seen by the left eye');
	
	if ~ischar(fileNameLeft) || ~ischar(filePathLeft) % User clicked cancel
		break;
	end
	
	[fileNameRight, filePathRight] = uigetfile(filterSpecCellArray, 'Select the image to be seen by the right eye');
	
	if ~ischar(fileNameLeft) || ~ischar(filePathLeft) % User clicked cancel
		break;
	end
	
	pathImgLeft = fullfile(filePathLeft, fileNameLeft);
	pathImgRight = fullfile(filePathRight, fileNameRight);

	imgLeft = imread(pathImgLeft);
	imgRight = imread(pathImgRight);
	
	if ~isequal(size(imgLeft), size(imgRight))
		errordlg('Left and right images are not the same size.', 'Error', 'modal');
		return
	end
	
	imgCombined = CombineImages(imgLeft, imgRight, false);
	
	hFig1 = figure;
	imshow(imgCombined);
	
	saveInverted = false;
	if strcmp(questdlg('Do you want to try inverting the source images?', 'Question', 'Yes', 'No', 'No'), 'Yes')
		imgCombinedInverted = CombineImages(imgLeft, imgRight, true);
		
		hFig2 = figure;
		imshow(imgCombinedInverted);
		
		if strcmp(questdlg('Do you prefer the inverted version to the original?', 'Question', 'Yes', 'No', 'No'), 'Yes')
			saveInverted = true;
		else
			close(hFig2);
		end
	end
	
	[fileNameSave, filePathSave] = uiputfile(filterSpecCellArray, 'Save result', [filePathLeft 'result.png']);
	
	if ~(isnumeric(fileNameSave) && fileNameSave == 0) % Destination file information is valid
		if saveInverted
			imwrite(imgCombinedInverted, fullfile(filePathSave, fileNameSave));
		else
			imwrite(imgCombined, fullfile(filePathSave, fileNameSave));
		end
		uiwait(msgbox(['Image saved to ' fullfile(filePathSave, fileNameSave)], 'Operation complete', 'modal'));
	end
	
	if ishandle(hFig1)
		close(hFig1)
	end
	
	if exist('hFig2', 'var')
		if ishandle(hFig2)
			close(hFig2)
		end
	end

end

	function result = CombineImages(imgLeftA, imgRightA, invert)
		imgLeftGray = double(rgb2gray(imgLeftA))/255;
		imgRightGray = double(rgb2gray(imgRightA))/255;

		if invert
			imgLeftGray = 1-imgLeftGray;
			imgRightGray = 1-imgRightGray;
		end

		imgColorRight = uint8(repmat(reshape(colorRight,[1 1 3]), [size(imgLeftA,1) size(imgLeftA,2)]));
		imgColorLeft = uint8(repmat(reshape(colorLeft,[1 1 3]), [size(imgLeftA,1) size(imgLeftA,2)]));
		imgColorBackground = uint8(repmat(reshape(colorBackground,[1 1 3]), [size(imgLeftA,1) size(imgLeftA,2)]));

		combined = double(imgColorBackground);

		% Blend in left eye view (regular blend)
		combined = combined.*(1 - repmat(imgLeftGray, [1 1 3])) + double(imgColorLeft).*repmat(imgLeftGray, [1 1 3]);

		% Blend in right eye view (darken blend)
		combined = combined.*(1 - repmat(imgRightGray, [1 1 3])) + min(combined, double(imgColorRight)).*repmat(imgRightGray, [1 1 3]);
		
		result = uint8(combined);
	end

end
